package com.nisum.reactiveweb.exceptions;

import java.util.Map;

import org.springframework.boot.web.error.ErrorAttributeOptions;
import org.springframework.boot.web.reactive.error.DefaultErrorAttributes;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.server.ServerRequest;

@Component
public class ApiError extends DefaultErrorAttributes {

    @Override
    public Map<String, Object> getErrorAttributes(ServerRequest request, ErrorAttributeOptions options) {
        Map<String, Object> errorAttributes = super.getErrorAttributes(request, options);
        Throwable error = getError(request);
        if (error instanceof ProductNotFoundException) {
            addErrorFields(errorAttributes, error, HttpStatus.NOT_FOUND);
        } else if (error instanceof IllegalArgumentException) {
            addErrorFields(errorAttributes, error, HttpStatus.BAD_REQUEST);
        } else {
            addErrorFields(errorAttributes, error, HttpStatus.INTERNAL_SERVER_ERROR);
        }
        return errorAttributes;
    }

    private void addErrorFields(Map<String, Object> errorAttributes, Throwable ex, HttpStatus status) {
        errorAttributes.put("message", ex.getMessage());
        errorAttributes.put("error", status);
        errorAttributes.put("status", status.value());
    }

}

