package com.nisum.reactiveweb.repository;

import com.mongodb.client.result.DeleteResult;
import com.nisum.reactiveweb.model.Product;
import lombok.AllArgsConstructor;
import reactor.core.publisher.Mono;

import org.springframework.data.mongodb.core.FindAndModifyOptions;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

@Repository
@AllArgsConstructor
public class ProductRepositoryImpl implements ProductRepository {

    private final ReactiveMongoTemplate template;

    @Override
    public Mono<Product> getProductBySku(String sku) {
        return template.findById(sku, Product.class);
    }

    @Override
    public Mono<Product> saveProduct(Product product) {
        return template.save(product);
    }

    @Override
    public Mono<Product> updateProduct(String sku, Product product) {
        return template.findAndModify(Query.query(Criteria.where("sku").is(sku)),
                                      Update.update("name", product.getName())
                                              .set("price", product.getPrice()),
                                      FindAndModifyOptions.options().returnNew(true),
                                      Product.class);
    }

    @Override
    public Mono<Long> removeProduct(String sku) {
        return template.remove(Query.query(Criteria.where("sku").is(sku)), Product.class)
                .map(DeleteResult::getDeletedCount);
    }

}
