package com.nisum.reactiveweb.service;

import com.nisum.reactiveweb.exceptions.ProductNotFoundException;
import com.nisum.reactiveweb.model.Product;
import com.nisum.reactiveweb.repository.ProductRepository;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

import org.springframework.stereotype.Service;

@Slf4j
@Service
@AllArgsConstructor
public class ProductServiceImpl implements ProductService {

    private final ProductRepository repository;

    public Mono<Product> getProductBySku(String sku) {
        return repository.getProductBySku(sku)
                .switchIfEmpty(Mono.error(new ProductNotFoundException()));
    }

    @Override
    public Mono<Product> saveProduct(Product productToSave) {
        return Mono.just(productToSave)
                .doOnNext(product -> product.setSku(null))
                .flatMap(repository::saveProduct);
    }

    @Override
    public Mono<Product> updateProduct(String sku, Product product) {
        return getProductBySku(sku)
                .then(repository.updateProduct(sku, product));
    }

    @Override
    public Mono<String> removeProduct(String sku) {
        return repository.removeProduct(sku)
                .flatMap(deletedCount -> deletedCount > 0 ? Mono.just("deleted") : Mono.error(new ProductNotFoundException()));
    }
}
